@@ -2,6 +2,7 @@
 import subprocess, sys
 import StringIO
 import inspect
+import types
 import os
 from threading import Thread
 
@@ -43,7 +44,8 @@ class CmdAction(BaseAction):
         self.task = None
         self.out = None
         self.err = None
-        self.value = None
+        self.result = None
+        self.values = {}
 
     def execute(self, out=None, err=None):
         """
@@ -90,7 +92,7 @@ class CmdAction(BaseAction):
 
         self.out = output.getvalue()
         self.err = errput.getvalue()
-        self.value = self.out + self.err
+        self.result = self.out + self.err
 
         # task error - based on:
         # http://www.gnu.org/software/bash/manual/bashref.html#Exit-Status
@@ -144,8 +146,7 @@ class Writer(object):
 class PythonAction(BaseAction):
     """Python action. Execute a python callable.
 
-    @ivar py_callable: (callable) Python callable that returns a boolean
-    result depending on the success of the action
+    @ivar py_callable: (callable) Python callable
     @ivar args: (sequence)  Extra arguments to be passed to py_callable
     @ivar kwargs: (dict) Extra keyword arguments to be passed to py_callable
     @ivar task(Task): reference to task that contains this action
@@ -156,7 +157,8 @@ class PythonAction(BaseAction):
         self.task = None
         self.out = None
         self.err = None
-        self.value = None
+        self.result = None
+        self.values = {}
 
         if args is None:
             self.args = []
@@ -241,7 +243,7 @@ class PythonAction(BaseAction):
                     a file like object (has write method)
         @param err: idem
 
-        @raise TaskFailed: If py_callable returns False
+        @raise TaskFailed: If py_callable returns False. or TaskError
         """
         # set std stream
         old_stdout = sys.stdout
@@ -265,7 +267,7 @@ class PythonAction(BaseAction):
         try:
             # Python2.4
             try:
-                result = self.py_callable(*self.args,**kwargs)
+                returned_value = self.py_callable(*self.args,**kwargs)
             # in python 2.4 SystemExit and KeyboardInterrupt subclass
             # from Exception.
             except (SystemExit, KeyboardInterrupt), exception:
@@ -279,19 +281,23 @@ class PythonAction(BaseAction):
             self.out = output.getvalue()
             self.err = errput.getvalue()
 
-        self.value = result
         # if callable returns false. Task failed
-        if result is False:
+        if returned_value is False:
             raise TaskFailed("Python Task failed: '%s' returned %s" %
-                             (self.py_callable, result))
-        elif result is True or result is None or isinstance(result, str):
-            return
+                             (self.py_callable, returned_value))
+        elif returned_value is True or returned_value is None:
+            pass
+        elif isinstance(returned_value, str):
+            self.result = returned_value
+        elif isinstance(returned_value, dict):
+            self.values = returned_value
         else:
             raise TaskError("Python Task error: '%s'. It must return:\n"
                             "False for failed task.\n"
-                            "True, None or string for successful task\n"
+                            "True, None, string or dict for successful task\n"
                             "returned %s (%s)" %
-                            (self.py_callable, result, type(result)))
+                            (self.py_callable, returned_value,
+                             type(returned_value)))
 
     def __str__(self):
         # get object description excluding runtime memory address
@@ -342,9 +348,12 @@ class Task(object):
     @ivar setup (list): List of setup objects
           (any object with setup or cleanup method)
     @ivar is_subtask: (bool) indicate this task is a subtask.
+    @ivar result: (str) last action "result". used to check task-result-dep
+    @ivar values: (dict) values saved by task that might be used by other tasks
+    @ivar getargs: (dict) values from other tasks
     @ivar doc: (string) task documentation
 
-    @ivar options: (dict) calculate params values
+    @ivar options: (dict) calculated params values (from getargs and taskopt)
     @ivar taskopt: (cmdparse.Command)
     @ivar custom_title: function reference that takes a task object as
                         parameter and returns a string.
@@ -362,17 +371,20 @@ class Task(object):
                   'doc': [str, None],
                   'params': [list, tuple],
                   'verbosity': [None,0,1,2],
+                  'getargs': [dict],
+                  'title': [None, types.FunctionType],
                   }
 
 
     def __init__(self, name, actions, dependencies=(), targets=(),
                  setup=(), clean=(), is_subtask=False, doc=None, params=(),
-                 verbosity=None, title=None):
+                 verbosity=None, title=None, getargs=None):
         """sanity checks and initialization
 
         @param params: (list of option parameters) see cmdparse.Command.__init__
         """
 
+        getargs = getargs or {} #default
         # check task attributes input
         for attr, valid_list in self.valid_attr.iteritems():
             self.check_attr_input(name, attr, locals()[attr], valid_list)
@@ -382,9 +394,11 @@ class Task(object):
         self.setup = setup
         self.run_once = False
         self.is_subtask = is_subtask
-        self.value = None #TODO document this
+        self.result = None
+        self.values = {}
         self.verbosity = verbosity
         self.custom_title = title
+        self.getargs = getargs
 
         # options
         self.taskcmd = cmdparse.TaskOption(name, params, None, None)
@@ -392,11 +406,13 @@ class Task(object):
         # options were passed on the command line.
         self.options = self.taskcmd.parse('')[0] # ignore positional parameters
 
+        # actions
         if actions is None:
             self.actions = []
         else:
             self.actions = [create_action(a) for a in actions]
 
+        # clean
         if clean is True:
             self._remove_targets = True
             self.clean_actions = ()
@@ -408,8 +424,14 @@ class Task(object):
         for action in self.actions:
             action.task = self
 
+        self._init_dependencies(dependencies)
+        self._init_getargs()
+        self._init_doc(doc)
+
+
+    def _init_dependencies(self, dependencies):
         self.dep_changed = None
-        # there are 2 kinds of dependencies: file, task
+        # there are 3 kinds of dependencies: file, task, result
         self.task_dep = []
         self.file_dep = []
         self.result_dep = []
@@ -419,7 +441,7 @@ class Task(object):
                 if not dep:
                     msg = ("%s. bool paramater in 'dependencies' "+
                            "must be True got:'%s'")
-                    raise InvalidTask(msg%(name, str(dep)))
+                    raise InvalidTask(msg%(self.name, str(dep)))
                 self.run_once = True
             # task dep starts with a ':'
             elif dep.startswith(':'):
@@ -433,12 +455,27 @@ class Task(object):
             elif isinstance(dep,str):
                 self.file_dep.append(dep)
 
+
+    def _init_getargs(self):
+        # getargs also define implicit task dependencies
+        for key, desc in self.getargs.iteritems():
+            # check format
+            parts = desc.split('.')
+            if len(parts) != 2:
+                msg = ("Taskid '%s' - Invalid format for getargs of '%s'.\n" %
+                       (self.name, key) +
+                       "Should be <taskid>.<argument-name> got '%s'\n" % desc)
+                raise InvalidTask(msg)
+            if parts[0] not in self.task_dep:
+                self.task_dep.append(parts[0])
+
         # run_once can't be used together with file dependencies
         if self.run_once and self.file_dep:
             msg = ("%s. task cant have file and dependencies and True " +
                    "at the same time. (just remove True)")
-            raise InvalidTask(msg % name)
+            raise InvalidTask(msg % self.name)
 
+    def _init_doc(self, doc):
         # Store just first non-empty line as documentation string
         if doc is None:
             self.doc = ''
@@ -496,10 +533,11 @@ class Task(object):
         task_stdout, task_stderr = VERBOSITY[use_verbosity]
         for action in self.actions:
             action.execute(task_stdout, task_stderr)
-            self.value = action.value
+            self.result = action.result
+            self.values.update(action.values)
 
 
-    def clean(self):
+    def clean(self, outstream, dryrun):
         """Execute task's clean"""
         # if clean is True remove all targets
         if self._remove_targets is True:
@@ -508,20 +546,30 @@ class Task(object):
 
             # remove all files
             for file_ in files:
-                os.remove(file_)
+                msg = "%s - removing file '%s'\n" % (self.name, file_)
+                outstream.write(msg)
+                if not dryrun:
+                    os.remove(file_)
 
             # remove all directories (if empty)
             for dir_ in dirs:
                 if os.listdir(dir_):
-                    msg = "Warning: '%s' cannot be removed (it is not empty)\n"
-                    sys.stderr.write(msg % dir_)
+                    msg = "%s - cannot remove (it is not empty) '%s'\n"
+                    outstream.write(msg % (self.name, file_))
                 else:
-                    os.rmdir(dir_)
+                    msg = "%s - removing dir '%s'\n"
+                    outstream.write(msg % (self.name, dir_))
+                    if not dryrun:
+                        os.rmdir(dir_)
 
         else:
             # clean contains a list of actions...
             for action in self.clean_actions:
-                action.execute()
+                msg = "%s - executing '%s'\n"
+                outstream.write(msg % (self.name, action))
+                if not dryrun:
+                    action.execute()
+
 
     def title(self):
         """String representation on output.
@@ -533,14 +581,6 @@ class Task(object):
         return self.name
 
 
-    def __str__(self):
-        if self.actions:
-            return "\n\t".join([str(action) for action in self.actions])
-
-        # A task that contains no actions at all
-        # is used as group task
-        return "Group: %s" % ", ".join(self.task_dep)
-
     def __repr__(self):
         return "<Task: %s>"% self.name
 
@@ -554,10 +594,6 @@ def dict_to_task(task_dict):
     @param task_dict (dict): task representation as a dict.
     @raise InvalidTask: If unexpected fields were passed in task_dict
     """
-    # TASK_ATTRS: sequence of know attributes(keys) of a task dict.
-    TASK_ATTRS = ('name','actions','dependencies','targets','setup', 'doc',
-                  'clean', 'params', 'verbosity', 'title')
-    # FIXME check field 'name'
 
     # check required fields
     if 'actions' not in task_dict:
@@ -566,7 +602,7 @@ def dict_to_task(task_dict):
 
     # user friendly. dont go ahead with invalid input.
     for key in task_dict.keys():
-        if key not in TASK_ATTRS:
+        if key not in Task.valid_attr.keys():
             raise InvalidTask("Task %s contains invalid field: '%s'"%
                               (task_dict['name'],key))
 
