@@ -22,13 +22,13 @@ TASKS_SAMPLE = [Task("t1", [""], doc="t1 doc string"),
                 Task("g1.a", [""], doc="g1.a doc string", is_subtask=True),
                 Task("g1.b", [""], doc="g1.b doc string", is_subtask=True),
                 Task("t3", [""], doc="t3 doc string")]
-
+TASKS_SAMPLE[2].task_dep = ['g1.a', 'g1.b']
 
 class TestCmdList(object):
 
     def testListTasksWithDoc(self):
         output = StringIO.StringIO()
-        doit_list(TASKS_SAMPLE, output, False)
+        doit_list(TESTDB, TASKS_SAMPLE, output, [], False, False, False)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = []
         for t in TASKS_SAMPLE:
@@ -38,25 +38,47 @@ class TestCmdList(object):
 
     def testListTasksWithDocQuiet(self):
         output = StringIO.StringIO()
-        doit_list(TASKS_SAMPLE, output, False, True)
+        doit_list(TESTDB, TASKS_SAMPLE, output, [], False, True, False)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = [t.name for t in TASKS_SAMPLE if not t.is_subtask]
         assert expected == got, output.getvalue()
 
     def testListAllTasksWithDoc(self):
         output = StringIO.StringIO()
-        doit_list(TASKS_SAMPLE, output, True)
+        doit_list(TESTDB, TASKS_SAMPLE, output, [], True, False, False)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = ["%s : %s" % (t.name, t.doc) for t in TASKS_SAMPLE]
-        assert expected == got, output.getvalue()
+        assert expected == got
 
     def testListAllTasksWithDocQuiet(self):
         output = StringIO.StringIO()
-        doit_list(TASKS_SAMPLE, output, True, True)
+        doit_list(TESTDB, TASKS_SAMPLE, output, [], True, True, False)
         got = [line for line in output.getvalue().split('\n') if line]
         expected = [t.name for t in TASKS_SAMPLE]
-        assert expected == got, output.getvalue()
+        assert expected == got
 
+    def testListFilter(self):
+        output = StringIO.StringIO()
+        doit_list(TESTDB, TASKS_SAMPLE, output, ['g1', 't2'],
+                  False, True, False)
+        got = [line for line in output.getvalue().split('\n') if line]
+        expected = ['g1', 't2']
+        assert expected == got
+
+    def testListFilterAll(self):
+        output = StringIO.StringIO()
+        doit_list(TESTDB, TASKS_SAMPLE, output, ['g1'], True, True, False)
+        got = [line for line in output.getvalue().split('\n') if line]
+        expected = ['g1', 'g1.a', 'g1.b']
+        assert expected == got
+
+    def testStatus(self):
+        output = StringIO.StringIO()
+        doit_list(TESTDB, TASKS_SAMPLE, output, ['g1'],
+                  False, True, True)
+        got = [line for line in output.getvalue().split('\n') if line]
+        expected = ['R g1']
+        assert expected == got
 
 
 class TestCmdForget(object):
@@ -116,7 +138,6 @@ class TestCmdForget(object):
     def testDontForgetTaskDependency(self, tasks):
         output = StringIO.StringIO()
         doit_forget(TESTDB, tasks, output, ["t3"])
-        got = output.getvalue().split("\n")[:-1]
         dep = Dependency(TESTDB)
         assert None == dep._get("t3", "dep")
         assert "1" == dep._get("t1", "dep")
@@ -191,12 +212,12 @@ class TestCmdClean(object):
 
     def test_clean_all(self, tasks):
         output = StringIO.StringIO()
-        doit_clean(self.tasks, output, [])
+        doit_clean(self.tasks, output, False, [])
         assert 2 == self.count
 
     def test_clean_selected(self, tasks):
         output = StringIO.StringIO()
-        doit_clean(self.tasks, output, ['t2'])
+        doit_clean(self.tasks, output, False, ['t2'])
 
 
 class TestCmdIgnore(object):
@@ -255,7 +276,6 @@ class TestCmdIgnore(object):
     def testDontIgnoreTaskDependency(self, tasks):
         output = StringIO.StringIO()
         doit_ignore(TESTDB, tasks, output, ["t3"])
-        got = output.getvalue().split("\n")[:-1]
         dep = Dependency(TESTDB)
         assert '1' == dep._get("t3", "ignore:")
         assert None == dep._get("t1", "ignore:")
