@@ -1,5 +1,6 @@
 import os, shutil
 import sys, tempfile
+from StringIO import StringIO
 
 import py.test
 
@@ -44,10 +45,16 @@ class TestCmdAction(object):
         expected = "<CmdAction: '%s'>" % PROGRAM
         assert  expected == repr(action), repr(action)
 
-    def test_value(self):
+    def test_result(self):
         action = task.CmdAction("%s 1 2" % PROGRAM)
         action.execute()
-        assert "12" == action.value
+        assert "12" == action.result
+
+    def test_values(self):
+        # for cmdActions they are always empty
+        action = task.CmdAction("%s 1 2" % PROGRAM)
+        action.execute()
+        assert {} == action.values
 
 
 class TestCmdVerbosity(object):
@@ -109,9 +116,15 @@ class TestPythonAction(object):
         # nothing raised it was successful
         action.execute()
 
+    def test_success_dict(self):
+        def success_sample():return {}
+        action = task.PythonAction(success_sample)
+        # nothing raised it was successful
+        action.execute()
+
     def test_error_object(self):
-        # anthing but None, bool, or string
-        def error_sample(): return {}
+        # anthing but None, bool, string or dict
+        def error_sample(): return object()
         action = task.PythonAction(error_sample)
         py.test.raises(TaskError, action.execute)
 
@@ -186,11 +199,17 @@ class TestPythonAction(object):
         action = task.PythonAction(repr_sample)
         assert  "<PythonAction: '%s'>" % repr(repr_sample) == repr(action)
 
-    def test_value(self):
+    def test_result(self):
         def vvv(): return "my value"
         action = task.PythonAction(vvv)
         action.execute()
-        assert "my value" == action.value
+        assert "my value" == action.result
+
+    def test_values(self):
+        def vvv(): return {'x': 5, 'y':10}
+        action = task.PythonAction(vvv)
+        action.execute()
+        assert {'x': 5, 'y':10} == action.values
 
 
 class TestPythonVerbosity(object):
@@ -328,7 +347,7 @@ class TestTask(object):
 
     def test_title(self):
         t = task.Task("MyName",["MyAction"])
-        assert "MyName" == t.title(), t.title()
+        assert "MyName" == t.name, t.name
 
 
     def test_custom_title(self):
@@ -336,11 +355,6 @@ class TestTask(object):
         assert "X%sX"%str(t.name) == t.title(), t.title()
 
 
-    def test_strGroup(self):
-        t = task.Task("taskX",None,('file_foo', ':t1',':t2'))
-        assert "Group: t1, t2" == str(t), "'%s'"%str(t)
-
-
     # dependency types going to the write place
     def test_dependencyTypes(self):
         dep = ["file1.txt",":taskX","file2", "?res1"]
@@ -359,17 +373,39 @@ class TestTask(object):
         assert '' == t.options['p2']
 
 
+class TestTask_Getargs(object):
+    def test_ok(self):
+        getargs = {'x' : 't1.x', 'y': 't2.z'}
+        t = task.Task('t3', None, getargs=getargs)
+        assert 't1' in t.task_dep
+        assert 't2' in t.task_dep
+
+    def test_invalid_desc(self):
+        getargs = {'x' : 't1'}
+        assert py.test.raises(task.InvalidTask, task.Task,
+                              't3', None, getargs=getargs)
+
+
 class TestTaskActions(object):
     def test_success(self):
         t = task.Task("taskX", [PROGRAM])
         t.execute()
 
 
-    def test_value(self):
-        #task.value is the value of last action
-        t = task.Task('t1', ["%s hi_list hi6" % PROGRAM,
-                             "%s hi_list hi6" % PROGRAM])
-        assert t.value == t.actions[-1].value
+    def test_result(self):
+        # task.result is the value of last action
+        t = task.Task('t1', ["%s hi_list hi1" % PROGRAM,
+                             "%s hi_list hi2" % PROGRAM])
+        t.execute()
+        assert "hi_listhi2" == t.result
+
+    def test_values(self):
+        def return_dict(d): return d
+        # task.result is the value of last action
+        t = task.Task('t1', [(return_dict, [{'x':5}]),
+                             (return_dict, [{'y':10}]),])
+        t.execute()
+        assert {'x':5, 'y':10} == t.values
 
     def test_failure(self):
         t = task.Task("taskX", ["%s 1 2 3" % PROGRAM])
@@ -433,7 +469,7 @@ class TestTaskClean(object):
         t = task.Task("xxx", None)
         assert False == t._remove_targets
         assert 0 == len(t.clean_actions)
-        t.clean()
+        t.clean(StringIO(), False)
         for filename in tmpdir['files']:
             assert os.path.exists(filename)
 
@@ -441,37 +477,33 @@ class TestTaskClean(object):
         t = task.Task("xxx", None, targets=tmpdir['files'], clean=True)
         assert True == t._remove_targets
         assert 0 == len(t.clean_actions)
-        t.clean()
+        t.clean(StringIO(), False)
         for filename in tmpdir['files']:
             assert not os.path.exists(filename), filename
 
     def test_clean_non_existent_targets(self):
         t = task.Task('xxx', None, targets=["i_dont_exist"], clean=True)
-        t.clean()
+        t.clean(StringIO(), False)
         # nothing is raised
 
     def test_clean_empty_dirs(self, tmpdir):
-        """
-        Remove empty directories listed in targets
-        """
+        # Remove empty directories listed in targets
         targets = tmpdir['files'] + [tmpdir['dir']]
         t = task.Task("xxx", None, targets=targets, clean=True)
         assert True == t._remove_targets
         assert 0 == len(t.clean_actions)
-        t.clean()
+        t.clean(StringIO(), False)
         for filename in tmpdir['files']:
             assert not os.path.exists(filename)
         assert not os.path.exists(tmpdir['dir'])
 
     def test_keep_non_empty_dirs(self, tmpdir):
-        """
-        Keep non empty directories listed in targets
-        """
+        # Keep non empty directories listed in targets
         targets = [tmpdir['files'][0], tmpdir['dir']]
         t = task.Task("xxx", None, targets=targets, clean=True)
         assert True == t._remove_targets
         assert 0 == len(t.clean_actions)
-        t.clean()
+        t.clean(StringIO(), False)
         for filename in tmpdir['files']:
             expected = not filename in targets
             assert expected == os.path.exists(filename)
@@ -487,7 +519,7 @@ class TestTaskClean(object):
         t = task.Task("xxx",None,targets=tmpdir['files'], clean=[(say_hello,)])
         assert False == t._remove_targets
         assert 1 == len(t.clean_actions)
-        t.clean()
+        t.clean(StringIO(), False)
         for filename in tmpdir['files']:
             assert os.path.exists(filename)
         fh = file(c_path, 'r')
@@ -495,6 +527,36 @@ class TestTaskClean(object):
         fh.close()
         assert "hello!!!" == got
 
+    def test_dryrun_file(self, tmpdir):
+        t = task.Task("xxx", None, targets=tmpdir['files'], clean=True)
+        assert True == t._remove_targets
+        assert 0 == len(t.clean_actions)
+        t.clean(StringIO(), True)
+        # files are NOT removed
+        for filename in tmpdir['files']:
+            assert os.path.exists(filename), filename
+
+    def test_dryrun_dir(self, tmpdir):
+        targets = tmpdir['files'] + [tmpdir['dir']]
+        for filename in tmpdir['files']:
+            os.remove(filename)
+        t = task.Task("xxx", None, targets=targets, clean=True)
+        assert True == t._remove_targets
+        assert 0 == len(t.clean_actions)
+        t.clean(StringIO(), True)
+        assert os.path.exists(tmpdir['dir'])
+
+    def test_dryrun_actions(self, tmpdir):
+        # a clean action can be anything, it can even not clean anything!
+        self.executed = False
+        def say_hello(): self.executed = True
+        t = task.Task("xxx",None,targets=tmpdir['files'], clean=[(say_hello,)])
+        assert False == t._remove_targets
+        assert 1 == len(t.clean_actions)
+        t.clean(StringIO(), True)
+        assert not self.executed
+
+
 
 class TestTaskDoc(object):
     def test_no_doc(self):
